# -*- coding: utf-8 -*-
"""WSGI environment setup for luci."""

from luci.config.app_cfg import base_config

__all__ = ['load_environment']


# This will enable removing some values from ``global_conf'' that had a use
# only for parsing + string interpolation of certain sections in config file;
# also values prefixed with "who." in ``app_conf'' are no longer needed
def make_load_environment(wrapped_f):

    def load_environment(global_conf, app_conf):
        # rhbz#982771
        secret_length = len(app_conf.get('beaker.session.secret', '').strip())
        if secret_length < 20:
            msg = ("Insufficient length of 'beaker.session.secret': {0}"
                   .format(secret_length))
            with open('/var/log/luci/luci.log', 'a') as f:
                f.write(msg.join(('\n', '\n')))
            raise RuntimeError(msg)

        for k in (i for i in global_conf.keys() if i.startswith('tmp.')
                                                or i.startswith('def.')):
            global_conf.pop(k)

        # Figure custom set of static HTTP headers (rhbz#1270958)
        headers = dict((k.split('header.', 1)[1].title(), app_conf.pop(k))
                        for k in app_conf.keys() if k.startswith('header.'))

        # Trigger wrapped/proper load_environment
        wrapped_f(global_conf, app_conf)

        # Enforce that set of static HTTP headers
        try:
            from pylons.configuration import response_defaults as _response_defaults
            response_options = _response_defaults.copy()
        except ImportError:
            response_options = dict(
                content_type='text/html',
                charset='utf-8',
                errors='strict',
                headers={'Cache-Control': 'no-cache',
                         'Pragma': 'no-cache'},
            )
        response_options['headers'] = response_options.get('headers', {}).copy()
        for k, v in headers.iteritems():
            if not v:
                response_options['headers'].pop(k, None)
            elif k not in response_options['headers']:  # leave defaults as are
                response_options['headers'][k] = v
        base_config.pylons_extra['response_options'] = response_options

        # Propagate the headers & more from pylons_extra to Pylons-backed cfg.
        from tg import config
        for key, item in base_config.pylons_extra.iteritems():
            config['pylons.{0}'.format(key)] = item

    # def load_environment

    return load_environment


load_environment = make_load_environment(base_config.make_load_environment())
