# -*- coding: utf-8 -*-
"""WSGI middleware initialization for the luci application."""

from luci.config.app_cfg import base_config
from luci.config.environment import load_environment
from luci.initwrappers import repoze_who_make_middleware_with_config


__all__ = ['make_app']

# Use base_config to setup the necessary PasteDeploy application factory.
# make_base_app will wrap the TG2 app with all the middleware it needs.
make_base_app = base_config.setup_tg_wsgi_app(load_environment)


def make_app(global_conf, full_stack=True, **app_conf):
    """
    Set luci up with the settings found in the PasteDeploy configuration
    file used.

    :param global_conf: The global settings for luci (those
        defined under the ``[DEFAULT]`` section).
    :type global_conf: dict
    :param full_stack: Should the whole TG2 stack be set up?
    :type full_stack: str or bool
    :return: The luci application with all the relevant middleware
        loaded.

    This is the PasteDeploy factory for the luci application.

    ``app_conf`` contains all the application-specific settings (those defined
    under ``[app:main]``.

    """
    # Figure out options specific for repoze.who middleware (~supported here)
    who_opts_defaults = dict(('who.' + k, v) for k, v in {
                                 'config_file': "who.ini",
                                 'log_file': None,
                                 'log_level': None,
                             }.iteritems())
    who_opts = dict(safe_config=False)
    who_opts.update((k.split('who.', 1)[1], app_conf.pop(k))
                    for k in app_conf.keys() if k in who_opts_defaults)
    who_conf = global_conf.copy()
    who_conf.update(app_conf)

    # Make the core application itself
    app = make_base_app(global_conf, full_stack=True, **app_conf)

    # Add repoze.who middleware
    app = repoze_who_make_middleware_with_config(app, who_conf, **who_opts)

    return app
