# Copyright (C) 2006-2015 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from TagObject import TagObject

TAG_NAME = 'totem'

class Totem(TagObject):
  DEFAULTS = {
    'join':                                 '60',
    'token':                                '10000',
    'fail_recv_const':                      '2500',
    'token_retransmits_before_loss_const':  '20',
    'consensus':                            '4800',
    'secauth':                              '1',
    'max_messages':                         '17',
    'netmtu':                               '1500',
    'window_size':                          '50',
    'seqno_unchanged_const':                '30',
  }

  def __init__(self):
    TagObject.__init__(self)
    self.TAG_NAME = TAG_NAME
    self.attr_hash.update(self.DEFAULTS)

  def getJoinTimeout(self):
    return self.getAttribute('join')

  def setJoinTimeout(self, val):
    return self.addIntegerAttribute('join', val, (0, None))

  def delJoinTimeout(self):
    return self.removeAttribute('join')

  def getTokenTimeout(self):
    return self.getAttribute('token')

  def setTokenTimeout(self, val):
    return self.addIntegerAttribute('token', val, (0, None))

  def delTokenTimeout(self):
    return self.removeAttribute('token')

  def getTokenRetransmits(self):
    return self.getAttribute('token_retransmits_before_loss_const')

  def setTokenRetransmits(self, val):
    return self.addIntegerAttribute('token_retransmits_before_loss_const', val, (0, None))

  def delTokenRetransmits(self):
    return self.removeAttribute('token_retransmits_before_loss_const')

  def getConsensusTimeout(self):
    return self.getAttribute('consensus')

  def setConsensusTimeout(self, val):
    return self.addIntegerAttribute('consensus', val, (0, None))

  def delConsensusTimeout(self):
    return self.removeAttribute('consensus')

  def getRRPMode(self):
    return self.getAttribute('rrp_mode')

  def setRRPMode(self, val):
    if not val:
        self.removeAttribute('rrp_mode')
        return None

    val = val.lower()
    if val not in ('active', 'passive', 'none'):
        raise ValueError, val
    self.addAttribute('rrp_mode', val)
    return val

  def delRRPMode(self):
    return self.removeAttribute('rrp_mode')

  def getSecAuth(self):
    return self.getBinaryAttribute('secauth')

  def setSecAuth(self, val):
    return self.addBinaryAttribute('secauth', val, ('off', 'on'))

  def delSecAuth(self):
    return self.removeAttribute('secauth')

  def getFailRecvConst(self):
    return self.getAttribute('fail_recv_const')

  def setFailRecvConst(self, val):
    return self.addIntegerAttribute('fail_recv_const', val, (0, None))

  def delFailRecvConst(self):
    return self.removeAttribute('fail_recv_const')

  def getKeyFile(self):
    return self.getAttribute('keyfile')

  def setKeyFile(self, val):
    return self.addAttribute('keyfile', val)

  def delKeyFile(self):
    return self.removeAttribute('keyfile')

  def getMaxMessages(self):
    return self.getAttribute('max_messages')

  def setMaxMessages(self, val):
    return self.addIntegerAttribute('max_messages', val, (0, None))

  def delMaxMessages(self):
    return self.removeAttribute('max_messages')

  def getNetMTU(self):
    return self.getAttribute('netmtu')

  def setNetMTU(self, val):
    return self.addIntegerAttribute('netmtu', val, (0, None))

  def delNetMTU(self):
    return self.removeAttribute('netmtu')

  def getWindowSize(self):
    return self.getAttribute('window_size')

  def setWindowSize(self, val):
    return self.addIntegerAttribute('window_size', val, (0, None))

  def delWindowSize(self):
    return self.removeAttribute('window_size')

  def getSeqnoUnchangedConst(self):
    return self.getAttribute('seqno_unchanged_const')

  def setSeqnoUnchangedConst(self, val):
    return self.addIntegerAttribute('seqno_unchanged_const', val, (0, None))

  def delSeqnoUnchangedConst(self):
    return self.removeAttribute('seqno_unchanged_const')

  def getRRPProblemCountThreshold(self):
    return self.getAttribute('rrp_problem_count_threshold')

  def setRRPProblemCountThreshold(self, val):
    return self.addIntegerAttribute('rrp_problem_count_threshold', val, (0, None))

  def delRRPProblemCountThreshold(self):
    return self.removeAttribute('rrp_problem_count_threshold')

  def getMissCountConst(self):
    return self.getAttribute('miss_count_const')

  def setMissCountConst(self, val):
    return self.addIntegerAttribute('miss_count_const', val, (0, None))

  def delMissCountConst(self):
    return self.removeAttribute('miss_count_const')

  def removeDefaults(self):
    if self.getJoinTimeout() == self.DEFAULTS.get('join'):
        self.delJoinTimeout()
    if self.getTokenTimeout() == self.DEFAULTS.get('token'):
        self.delTokenTimeout()
    if self.getTokenRetransmits() == self.DEFAULTS.get('token_retransmits_before_loss_const'):
        self.delTokenRetransmits()
    if self.getConsensusTimeout() == self.DEFAULTS.get('consensus'):
        self.delConsensusTimeout()
    if self.getSecAuth() == int(self.DEFAULTS.get('secauth')):
        self.delSecAuth()
    if self.getFailRecvConst() == self.DEFAULTS.get('fail_recv_const'):
        self.delFailRecvConst()
    if self.getRRPMode() == self.DEFAULTS.get('rrp_mode'):
        self.delRRPMode()
    if self.getMaxMessages() == self.DEFAULTS.get('max_messages'):
        self.delMaxMessages()
    if self.getNetMTU() == self.DEFAULTS.get('netmtu'):
        self.delNetMTU()
    if self.getWindowSize() == self.DEFAULTS.get('window_size'):
        self.delWindowSize()
    if self.getSeqnoUnchangedConst() == self.DEFAULTS.get('seqno_unchanged_const'):
        self.delSeqnoUnchangedConst()
