# Copyright 2014 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

# -*- coding: utf-8 -*-
"""Helpers used in Luci."""

__all__ = ['singleton',
           'relativeUrlSlashPrefix', 'relativeUrlList2Str']

from pylons.i18n import ugettext, lazy_ugettext
from urllib import (quote_plus as quote_plus_impl,
                   unquote_plus as unquote_plus_impl)


def singleton(cls):
    """Trivial class decorator that replace the class with an instance of it."""
    return cls()


#-------------------------------------------------------------------------------
# Functions related to URL.

def relativeUrlSlashPrefix(url_string):
    """Ensures that given URL (as a string) starts with a slash.

    Keyword arguments:
        url_string    URL (as a string).

    Return value(s):
        'url_string' argument surely starting with a slash.

    """
    if not url_string.startswith(u'/'):
        url_string = u'/' + url_string
    return url_string


def relativeUrlList2Str(*url_parts):
    """Converts list of URL parts to string beginning with '/'.

    Keyword arguments:
        url_parts    List of URL parts.

    Return value(s):
        All URL parts joined with a slash, the whole such string starts also
        with a slash.

    """
    return relativeUrlSlashPrefix(u'/'.join(url_parts))


#-------------------------------------------------------------------------------
# Functions related to HTML entities un/quoting.

def quote_plus(s):
    """urllib's quote_plus, but works with unicode as well."""
    return quote_plus_impl(s.encode('UTF-8') if isinstance(s, unicode) else s)


def unquote_plus(s):
    """urllib's unquote_plus, but return unicode."""
    ret = unquote_plus_impl(s)
    return ret if isinstance(ret, unicode) else unicode(ret, 'UTF-8')


def unquote_plus_term_traverse(resp_ext, ps):
    """Normalize single traversal-like entity at URL tail (no such -> None)"""
    resp_ext = resp_ext or u''
    return '/'.join([unquote_plus(p) for p in ps]) + resp_ext if ps else None
