# -*- coding: utf-8 -*-
"""Setup the luci application"""

__all__ = ['setup_app']


# This is a way how to inject logger file-like object from non-serviceable
# part of the configuration as a backend for standard and error outputs.
# Main motivation is to seamlessly switch from naive file-opened-once
# default # behavior of paster's LazyWriter to the logrotate-aware one
# (logging.handlers.WatchedFileHandler) should this be relevant, which
# it commonly is (rhbz#887340).
def _inject_target_logger_to_stdio():
    import sys
    from logging import FileHandler, root
    from paste.script.serve import LazyWriter

    class WatchedLazyWriter(LazyWriter):
        """paste.script.serve.LazyWriter working with WatchedFileHandler

        ...from logging.handlers suite"""
        def __init__(self, logger):
            file_handler = None
            for h in reversed(logger.handlers):
                if isinstance(h, FileHandler):
                    file_handler = h
                    break
            else:
                raise RuntimeError("no file handler for logger")
            LazyWriter.__init__(self, file_handler.baseFilename)
            self.logger = logger
            self.file_handler = file_handler

        def open(self):
            if not self.file_handler.stream:
                self.logger.warn("--enforced logging stream refresh--")
            if not self.file_handler.stream:
                raise RuntimeError("cannot refresh logging stream")
            return self.file_handler.stream

    lw = WatchedLazyWriter(root)
    sys.stdout = sys.stderr = lw


def setup_app(command, conf, vars):
    """Place any commands to setup luci here"""

    import transaction
    from tg import config

    from luci.config.environment import load_environment

    # This is analogous to use in initwrappers.server_runner,
    # but this time for "paste setup-app".
    _inject_target_logger_to_stdio()

    load_environment(conf.global_conf, conf.local_conf)
    # Load the models
    from luci import model
    print "Creating tables"
    model.metadata.create_all(bind=config['pylons.app_globals'].sa_engine)

    group = model.Group()
    group.group_name = u'managers'
    group.display_name = u'Administrators'

    model.DBSession.add(group)

    create_group = model.Group()
    create_group.group_name = u'create_cluster'
    create_group.display_name = u'May Create New Clusters'
    model.DBSession.add(create_group)

    import_group = model.Group()
    import_group.group_name = u'import_cluster'
    import_group.display_name = u'May Import Existing Clusters'
    model.DBSession.add(import_group)

    permission = model.Permission()
    permission.permission_name = u'manage'
    permission.description = u'This permission gives an administrative right to the bearer'
    permission.groups.append(group)

    model.DBSession.add(permission)
    model.DBSession.flush()
    transaction.commit()
    print "Successfully setup"
